<?php

namespace Quivers\Integration\Observer;

use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\Event\Observer as EventObserver;
use Psr\Log\LoggerInterface as Logger;
use Magento\Framework\Exception\CouldNotSaveException;

class ConfigObserver implements ObserverInterface
{
    /**
     * @var Logger
     */
    protected $logger;

    protected $curlClient;
    protected $scopeConfig;
    protected $configWriter;
    protected $configValueFactory;

    /**
     * @param Logger $logger
     */
    
    const XML_MARKETPLACE_ID = 'quivers/parameters/marketplaceId';
    const XML_PRODUCTGROUP_ID = 'quivers/parameters/ProductGroupId';
    const XML_QUIVERS_API_KEY = 'quivers/parameters/quivers_api_key';
    const XML_QUIVERS_BUSINESS_REFID = 'quivers/parameters/merchant_ref_id';
    const XML_MAGENTO_ACCESS_TOKEN = 'quivers/parameters/magento_access_token';
    const XML_MAGENTO_BASE_URL = 'quivers/parameters/magento_base_url';
    const XML_QUIVERS_SANDBOX = 'quivers/parameters/quivers_sanndbox';
    const XML_USD_MARKETPLACE_ID = 'quivers_tax/marketplaces/usdMarketPlaceId';
    const XML_EURO_MARKETPLACE_ID = 'quivers_tax/marketplaces/euroMarketPlaceId';
    const XML_CAD_MARKETPLACE_ID = 'quivers_tax/marketplaces/cadMarketPlaceId';
    const XML_AUD_MARKETPLACE_ID ='quivers_tax/marketplaces/audMarketPlaceId';
    const XML_JPY_MARKETPLACE_ID = 'quivers_tax/marketplaces/jpyMarketPlaceId';
    const XML_GBP_MARKETPLACE_ID = 'quivers_tax/marketplaces/gbpMarketPlaceId';
    const XML_QUIVERS_UUID = 'quivers/parameters/quivers_uuid';


    protected $baseUrl = 'https://middleware.quiversdemo.com/profile';


    public function __construct(
        Logger $logger,
        \Magento\Framework\HTTP\Client\Curl $curl,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Framework\App\Config\Storage\WriterInterface $configWriter,
        \Magento\Framework\App\Config\ValueFactory $configValueFactory
    ) {
        $this->curlClient = $curl;
        $this->logger = $logger;
        $this->scopeConfig = $scopeConfig;
        $this->configWriter = $configWriter;
        $this->configValueFactory = $configValueFactory;
    }

    public function execute(EventObserver $observer)
    {
        $storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        $marketPlaceId = $this->scopeConfig->getValue(self::XML_MARKETPLACE_ID, $storeScope);
        $usdMarketPlaceId = $this->scopeConfig->getValue(self::XML_USD_MARKETPLACE_ID, $storeScope);
        $euroMarketPlaceId = $this->scopeConfig->getValue(self::XML_EURO_MARKETPLACE_ID, $storeScope);
        $cadMarketPlaceId = $this->scopeConfig->getValue(self::XML_CAD_MARKETPLACE_ID, $storeScope);
        $audMarketPlaceId = $this->scopeConfig->getValue(self::XML_AUD_MARKETPLACE_ID, $storeScope);
        $jpyMarketPlaceId = $this->scopeConfig->getValue(self::XML_JPY_MARKETPLACE_ID, $storeScope);
        $gbpMarketPlaceId = $this->scopeConfig->getValue(self::XML_GBP_MARKETPLACE_ID, $storeScope);
        $productGroupId = $this->scopeConfig->getValue(self::XML_PRODUCTGROUP_ID, $storeScope);
        $quiversAPIKey = $this->scopeConfig->getValue(self::XML_QUIVERS_API_KEY, $storeScope);
        $magentoBaseUrl = $this->scopeConfig->getValue(self::XML_MAGENTO_BASE_URL, $storeScope);
        $quiversSandbox = $this->scopeConfig->getValue(self::XML_QUIVERS_SANDBOX, $storeScope);
        $quiversBusinessRefId = $this->scopeConfig->getValue(
            self::XML_QUIVERS_BUSINESS_REFID, $storeScope);
        $magentoAcessToken = $this->scopeConfig->getValue(
            self::XML_MAGENTO_ACCESS_TOKEN, $storeScope);

        if($quiversSandbox == 0) {
            $this->baseUrl = "https://middleware.quivers.com/profile";
        }

        $profileUrl = $this->baseUrl . "/create";
        $profileUpdateUrl = $this->baseUrl . "/update/marketplaces";

        $product_group_id_array = explode(", ", $productGroupId);
        $product_group_id_list = [];
        foreach ($product_group_id_array as $productGroupId) {
            array_push($product_group_id_list, $productGroupId);
        }

        $payload = [
            "client_type" => "Magento2",
            "base_url" => $magentoBaseUrl,
            "business_refid" => $quiversBusinessRefId,
            "api_key" => $quiversAPIKey,
            "auth_token" => "Bearer ".$magentoAcessToken
        ];
        $this->curlClient->addHeader("Content-Type", "application/json");
        $this->curlClient->post($profileUrl, json_encode($payload));

        $response = $this->curlClient->getBody();
        $allMarketPlaces = [];

        $marketPlaceIds = [
            "USD" => $usdMarketPlaceId,
            "CAD" => $cadMarketPlaceId,
            "EURO" => $euroMarketPlaceId,
            "AUD" => $audMarketPlaceId,
            "JPY" => $jpyMarketPlaceId,
            "GBP" => $gbpMarketPlaceId
        ];
        foreach($marketPlaceIds as $currency=>$mPlaceId) {
            if($mPlaceId) {
                $allMarketPlaces[] = [
                    'marketplace_id' => $mPlaceId,
                    'currency' => $currency,
                    'claiming_group_ids' => $product_group_id_list         
                ];
            }
        }
        if($this->curlClient->getStatus()==200) {
            $response = json_decode($response, true);
            $uuid = $response["uuid"];
            if($uuid) {
                $save = $this->configWriter->save(self::XML_QUIVERS_UUID, $uuid);
            }
            if(count($allMarketPlaces)>0) {
                $payload = [ "marketplaces" => $allMarketPlaces ];
                $this->curlClient->addHeader("uuid", $uuid);
                $this->curlClient->post($profileUpdateUrl, json_encode($payload));
                $response = $this->curlClient->getBody();
                if($this->curlClient->getStatus()>=400) {
                    $errorMessage = 'An error occurred while updating the profile. ERROR: '.$response;
                    throw new CouldNotSaveException(
                        __($errorMessage));
                }
            }
        } else {
            $errorMessage = 'An error occurred while creating the profile. ERROR: '.$response;
            throw new CouldNotSaveException(
                __($errorMessage));
        }
    }
}
